Class: SingleFunctionGraph
- Inherits:
-
FunctionGraph
- Object
- RGL::DirectedAdjacencyGraph
- FunctionGraph
- SingleFunctionGraph
- Defined in:
- lib/codegraph.rb
Direct Known Subclasses
Constant Summary
Constants inherited from FunctionGraph
Instance Attribute Summary collapse
-
#func ⇒ Object
Returns the value of attribute func.
Attributes inherited from FunctionGraph
#adds, #debug, #excludes, #funx
Instance Method Summary collapse
-
#fill(filelist, exclude = []) ⇒ Object
Works like the one from FunctionGraph except, that it calls ‘scan’ for the recursive descent throug all functions given in <filelist> - exclude.
-
#initialize(func) ⇒ SingleFunctionGraph
constructor
Constructor, which creates an empty graph for the rootfunction <func>.
Methods inherited from FunctionGraph
#display, #display_functionbody, #limit, #to_dot, #to_txt, #to_type
Constructor Details
#initialize(func) ⇒ SingleFunctionGraph
Constructor, which creates an empty graph for the rootfunction <func>
225 226 227 228 229 230 231 |
# File 'lib/codegraph.rb', line 225 def initialize(func) # Holds the func'n names, that are allready scanned @scannednames = [] # Root func @func = func super() end |
Instance Attribute Details
#func ⇒ Object
Returns the value of attribute func.
222 223 224 |
# File 'lib/codegraph.rb', line 222 def func @func end |
Instance Method Details
#fill(filelist, exclude = []) ⇒ Object
Works like the one from FunctionGraph except, that it calls ‘scan’ for the recursive descent throug all functions given in <filelist> - exclude
235 236 237 238 |
# File 'lib/codegraph.rb', line 235 def fill(filelist,exclude=[]) genFiles(self,filelist,exclude) scan(self, func) end |